@@ -11,7 +11,7 @@ hover_reply <- function(id, uri, workspace, document, position) {
     logger$info("hover: ", hover)
 
     matches <- stringr::str_match(
-        hover, "(?:([a-zA-Z][a-zA-Z0-9]+)(:::?))?([a-zA-Z0-9_.]*)$")
+        hover, "(?:([a-zA-Z][a-zA-Z0-9.]+)(:::?))?([a-zA-Z0-9_.]*)$")
 
     contents <- tryCatch(
         workspace$get_help(matches[4], matches[2]),
